# @file journal.pl
#!/usr/bin/perl
# @author nas
# @date 31/01/2022
# @brief Remote call to the API in order to test

use Config::Std;
use LWP::UserAgent ();
use lib qw(..);
use JSON qw( );
use Data::Dumper;
use Term::ANSIColor;

read_config 'test.cfg' => my %config; 

local $url = "$config{Authentification}{Protocol}$config{Authentification}{Host}$config{Authentification}{Path}";
local $ua = LWP::UserAgent->new(timeout => 10);
$ua->env_proxy;

my $count_success = 0;
my $count_failure = 0;

sub print_color {
  my $color = (exists $_[0])? $_[0] : "";
  my $text = (exists $_[0])? $_[1] : "";
  print color( "$color" );
  print "$text";
  print color('reset')
}

sub record_result {
  my $status = (exists $_[0])? $_[0] : false;

  if( $status ){
    print_color( "green", "Success :");
    $count_success++;
  }else{
    print_color( "red", "Failure :");
    $count_failure++;
  }
}

sub batch_brief {
    print "\nResults :\n";
    print_color( "bright_green", "Tests succeed : $count_success\n" );
    print_color( "bright_red", "Tests failed  : $count_failure\n" );
}

sub get_code {
  my $path = (exists $_[0])? $_[0] : "";
  my $response = $ua->get( "$url$path" );

  if ($response->is_success) {
    return $response->decoded_content;
  }
  else {
    return $response->status_line;
  }
}

sub assertEqual {
  my $value1 = (exists $_[0])? $_[0] : "";
  my $value2 = (exists $_[1])? $_[1] : "";
  my $equal = (exists $_[1])? $_[1] : 1;
  my $success = 0;

  my $test = ( $value1 eq $value2 );
  $test = ( $equal )? $test : !$test;

  if( $test )
  {
    record_result( 1 );
    print " $value1 ";
    print_color( "bright_blue", "equal");
    print " $value2\n";
  }else{
    record_result( 0 );
    print " $value1 ";
    print_color( "bright_blue", "NOT equal");
    print " $value2\n";
  };
}

sub assertNotEqual {
    assertEqual( $_[0], $_[1], 0 );
}

sub rq {
  my $path = (exists $_[0])? $_[0] : "";
  my $method = (exists $_[1])? $_[1] : "";
  my $content = (exists $_[2])? $_[2] : "";

  my $req = HTTP::Request->new("$method" => "$url$path");
  $req->content_type('application/json');
  $req->content("$content");
  my $res = $ua->request($req);
  return $res->decoded_content;
  return $res->as_string;
}

sub json_parse {
  my $json_text = (exists $_[0])? $_[0] : "";
  my $json = JSON->new;
  return $json->decode($json_text);
}

print "\nMaking all tests\n";
assertEqual( get_code(), "404 Not Found" );

my $result = json_parse( rq( "tokens", "POST", "{\"user\" : {\"email\" : \"$config{Authentification}{Email}\", \"password\" : \"$config{Authentification}{Password}\"}}" ));
assertEqual( "$config{Authentification}{Email}", $result->{"email"});
batch_brief();
